#  Copyright 2009, Erik Hahn

#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


import warnings
warnings.filterwarnings("ignore", message="the md5 module is deprecated; " + \
        "use hashlib instead")

import fnmatch
import os
import sys

from tag_wrapper import tag, TagException

# Some extensions might be missing
ACCEPTEXTS = [
        ".asf", ".wma", ".wmv",     # ASF, Windows Media
        ".flac",                    # FLAC
        ".mp3",                     # MP3
        ".ape",                     # Monkey's audio
        ".mp4", ".m4a",
        ".mpc", ".mpp", ".mp+",     # Musepack (APEv2)
        ".oga", ".ogg",             # Ogg Vorbis
        ".tta",                     # True Audio (hopefully)
        ".wv", ".wvc"               # WavPack
        ]

OPTIONS = {}


def gettext(domain):
    """
    Customized function to install gettext

    Based on <http://tinyurl.com/3ndgpy>
    """
    import gettext, locale

    localedir = os.path.join(os.path.dirname(sys.argv[0]), "locale")
    langs = []
    # This one should be defined on Windows
    lc = locale.getdefaultlocale()[0]
    if (lc):
        langs += [lc]
    # This one usually isn't
    language = os.environ.get("LANGUAGE", None)
    if (language):
        langs += language.split(":")

    trans = gettext.translation(domain, localedir, languages=langs,
            fallback=True)
    return trans.gettext


def issupportedfile(name):
    for ext in ACCEPTEXTS:
        if os.path.splitext(name)[1] == ext:
            return True
    return False


def newpath(file, pattern, windows=False):
    # Create the tokens dictionary
    tokens = {}
    tags = tag(file)
    # TODO: add tracknumber, disknumber and possibily compilation
    for t in ["title", "artist", "album artist", "album", "composer", "genre", "date"]:
        try:
            tokens[t] = tags[t]
        except KeyError:
            tokens[t] = None
    # %album artist% is %artist% if nothing can be found in the tags
    if tokens["album artist"] is None:
        tokens["album artist"] = tokens["artist"]

    # Now replace all tokens by their values
    for i in tokens:
        repl = "%" + i + "%"
        if tokens[i] is not None:
            if windows:
                val = sanitize_path(tokens[i][0])
            else:
                val = tokens[i][0]
            pattern = pattern.replace(repl, val)

    if windows:
        # Windows doesn't accept trailing dots in filenames
        t = pattern.split(os.sep)
        if os.name == "nt":
            npath = t[0] + '\\'
        else:
            npath = t[0]
        for j in t[1:-1]:
            npath = os.path.join(npath, (j.rstrip('.')))
        npath = os.path.join(npath, t[-1])
    else:
        npath = pattern
        

    # Add the extension and return the new path
    return npath + os.path.splitext(file)[1]


def safeprint(string):
    """
    Print string first trying to normally encode it, sending it to the
    console in "raw" UTF-8 if it fails.

    This is a workaround for Windows's broken console
    """
    try:
        print string
    except UnicodeEncodeError:
        print string.encode("utf-8")


def sanitize_path(string):
    # replaces all characters invalid in windows path and file names by '+'
    for i in ['/', '\\', ':', '*', '?', '"', '<', '>', '|']:
        string = string.replace(i, '+')
    return string


def samefile(file1, file2):
    # samefile() replacement for windows. This implementation will fail
    # in more complicated cases (junctions, network shares, mounts, etc.)
    if os.name == "posix":
        return os.path.samefile(file1, file2)
    else:
        file1 = os.path.normcase(os.path.realpath(file1))
        file2 = os.path.normcase(os.path.realpath(file2))
        return file1 == file2


# Begin tests
import unittest

class NewpathTestCase(unittest.TestCase):
    def testtags(self):
        file = os.path.join("..", "test", "sample.ogg")
        pattern = "%title% - %artist% - %album artist% - %album% - %composer% - %genre% - %date%"
        expect = "Title - Artist - Album Artist - Album - Composer - Genre - 2001.ogg"
        result = newpath(file, pattern)
        self.assertEqual(result, expect)

if __name__ == "__main__":
    unittest.main()
